package de.mach.tools.neodesigner.database;

import de.mach.tools.neodesigner.core.datamodel.ForeignKey;
import de.mach.tools.neodesigner.core.datamodel.Table;

import java.util.List;

/**
 * Interface mit dem andere Schichten auf die Datenbank zugreifen knnen
 * 
 * @author Chris Deter
 *
 */
public interface DatabaseConnection extends SaveDatabase {
  /**
   * gibt zurck ob die Datenbank Online ist.
   *
   * @return True wenn Online
   */
  boolean isOnline();

  /**
   * verbindet die Datebank.
   *
   * @param addr
   *          Adresse der Datenbank
   * @param nutzername
   *          der Nutzername
   * @param pw
   *          das Passwort
   * @return True wenn die Verbindung erfolgreich war
   */
  boolean connectDb(String addr, String nutzername, String pw);

  /**
   * Trennt die Datenbankverbindung.
   */
  void disconnectDb();

  /**
   * erstellt einen Index in der Graphendatenbank fr beschleunigten Zugriff.
   */
  void createIndexOnDb();

  /**
   * Fgt eine Tabelle mit allen Feldern in der Datenbank ein.
   *
   * @param t
   *          Die Tabelle
   */
  void importTable(Table t);

  /**
   * fgt eine Fremdschlsselbeziehung zwischen Table->RefTable und.
   * Fremdschlssel->RefTable-XPK in die Datenbank ein.
   *
   * @param i
   *          der Fremdschlssel
   */
  void importForeignKey(ForeignKey i);

  /**
   * gibt eine String Liste fr das Autocomplete Feld zurck.
   *
   * @return String Liste mit allen Tabellennamen
   */
  List<String> getListWithTableNames();

  /**
   * Gibt eine Tabelle aus der Datenbank zurck.
   *
   * @param name
   *          der Tabellename
   * @return die Tabelle
   */
  @Override
  Table getTable(String name);

  /**
   * Gibt eine Liste mit allen Tabellen zurck (WARNUNG: bei groen Datenmengen.
   * recht langsam!)
   *
   * @return gibt eine Liste mit allen Tabellen zurck
   */
  List<Table> getAllTables();

  /**
   * gibt eine Nummer fr den letzten ForeignKey Schlssel zurck.
   *
   * @param lenght
   *          Die Lnge der Nummer
   * @return neue FK Nummer
   */
  int getForeignKeyNumber(int lenght);

  /**
   * Lscht die Datenbank.
   *
   * @return true wenn erfolgreich
   */
  boolean deleteDatabase();
}